type ZodType<T> = {
    parse: (data: any) => T;
    array: () => ZodType<T[]>;
};
type parseOptions<T> = {
    default?: T;
    parseHandler?: (value: DataType) => T;
};
type DataRecord = {
    [P in string]: DataType;
};
type DataType = DataRecord | DataRecord[] | any[] | string | number | boolean | null | undefined;
type Handler<T> = (row: Inferer) => T;
type Inferer = ReturnType<typeof infer>;
declare function infer(data: DataType): {
    map: <T>(path: string | undefined, handler: Handler<T>) => T[];
    array: <T_1 = DataType>(path: string, schema: ZodType<T_1> | Handler<T_1>, options?: parseOptions<T_1[]>) => T_1[];
    object: <T_2>(path: string, schema: ZodType<T_2> | Handler<T_2>, options?: parseOptions<T_2>) => T_2;
    string: (path?: string, options?: parseOptions<string>) => string;
    number: (path?: string, options?: parseOptions<number> & {
        float?: boolean;
    }) => number;
    boolean: (path?: string, options?: parseOptions<boolean>) => boolean;
    nullable: (path?: string) => {
        string: (options?: parseOptions<string>) => string | null;
        number: (options?: parseOptions<number> & {
            float?: boolean;
        }) => number | null;
        boolean: (options?: parseOptions<boolean>) => boolean | null;
        stringOrNumber: (options?: parseOptions<string | number> & {
            float?: boolean;
        }) => string | number | null;
        array: <T_3 = DataType>(handler: ZodType<T_3> | Handler<T_3>, options?: parseOptions<T_3[]>) => T_3[] | null;
        object: <T_4>(handler: ZodType<T_4> | Handler<T_4>, options?: parseOptions<T_4>) => T_4 | null;
        map: <T_5>(handler: Handler<T_5>) => T_5[] | null;
    };
    get: (path?: string) => DataType;
    stringOrNumber: (path?: string, options?: parseOptions<string | number> & {
        float?: boolean;
    }) => string | number;
    readonly raw: DataType;
    has(path: string): boolean;
};

interface Schema {
    [K: string]: Schema | string;
}
type ScopePrefixer<T> = T extends infer K ? K extends string ? K extends '' ? '' : `${K}.` : never : never;
type UnionScopes<T, Prefix extends string = ''> = T extends Record<string, any> ? {
    [K in keyof T]: K extends string ? T[K] extends Record<string, any> ? UnionScopes<T[K], `${ScopePrefixer<Prefix>}${K}`> : `${ScopePrefixer<Prefix>}${K}` : never;
}[keyof T] : Prefix;
type PermitNode = {
    scope: string;
    granted: boolean;
    desc: string;
    toggle: () => void;
};
type TransformSchema<T, TValue> = T extends Schema ? {
    [P in keyof T]: T[P] extends Schema ? TransformSchema<T[P], TValue> : TValue;
} : TValue;
type SchemaOptions = {
    /** add prefix to scope. default to '' (none) */
    prefix: string;
    /** string to use for spacer for scope. default to '.' */
    spacer: string;
};

type SchemaNode = {
    scope: string;
    description: string;
};
declare class AccessSchema<T extends Schema = Schema> {
    private _options;
    private _schema;
    private _scopes;
    constructor(schema: T, options?: Partial<SchemaOptions>);
    private createSchemaNode;
    private resetSchemaNode;
    private get scopesList();
    /** utility function for finding scope */
    private _get;
    /** readonly schema options */
    get options(): SchemaOptions;
    /** readonly AccessNodes  */
    get nodes(): TransformSchema<T, string>;
    /** export schema to list of scope and description */
    export(): SchemaNode[];
    /** get schema node by string scope */
    get(scope: string): SchemaNode | undefined;
    /** utility function for checking if there is a schema node that have an exact match with given scope */
    has(scope: string): boolean;
    /** find schema node that has the given substring in scope or description. */
    search(substring: string, fn?: (n: SchemaNode) => boolean): SchemaNode[];
    /** find all child of the given string scope */
    getChild(scope: string): SchemaNode[];
    /** Map over schema and transform leaf node by given handler */
    map<V>(handler: (node: SchemaNode) => V): TransformSchema<T, V>;
    /**
     * Create permit from schema with given granted scopes
     */
    permit(scopes: string[] | SchemaNode[]): {
        get: (scope: UnionScopes<T, "">) => PermitNode | undefined;
        has: (scope: UnionScopes<T, "">) => boolean;
        search: (substring: string, handle: (node: PermitNode) => void) => void;
        getChild: (scope: string) => PermitNode[];
        reset: () => void;
        grant: (...scopes: string[] | SchemaNode[]) => void;
        deny: (...scopes: string[] | SchemaNode[]) => void;
        check(scope: string): {
            readonly exact: boolean;
            readonly hasChild: boolean;
            readonly hasAnyChildGranted: boolean;
            readonly hasAllChildGranted: boolean;
            readonly hasAllChildDenied: boolean;
        };
        readonly export: {
            readonly all: {
                scope: string;
                desc: string;
            }[];
            readonly granted: {
                scope: string;
                desc: string;
            }[];
            readonly denied: {
                scope: string;
                desc: string;
            }[];
        };
        readonly granted: {
            scope: string;
            desc: string;
        }[];
        readonly denied: {
            scope: string;
            desc: string;
        }[];
        readonly nodes: TransformSchema<T, PermitNode>;
        readonly can: TransformSchema<T, boolean>;
    };
}

/**
 * Create access control schema
 *
 * @param {T} schema
 * @param {Partial<SchemaOptions>} [options={}]
 * @return {AccessSchema} access schema
 */
declare function createSchema<T extends Schema>(schema: T, options?: Partial<SchemaOptions>): AccessSchema<T>;
type PermitOptions = {
    granted: string[] | SchemaNode[];
    denied: string[] | SchemaNode[];
} & SchemaOptions;
/**
 * Create user permit
 *
 * @export
 * @template T
 * @param {AccessSchema<T>} schema
 * @param {(string[] | SchemaNode[])} scopes
 * @return {Permit}
 */
declare function createPermit<T extends Schema>(schema: T, options?: Partial<PermitOptions>): {
    get: (scope: UnionScopes<T, "">) => PermitNode | undefined;
    has: (scope: UnionScopes<T, "">) => boolean;
    search: (substring: string, handle: (node: PermitNode) => void) => void;
    getChild: (scope: string) => PermitNode[];
    reset: () => void;
    grant: (...scopes: string[] | SchemaNode[]) => void;
    deny: (...scopes: string[] | SchemaNode[]) => void;
    check(scope: string): {
        readonly exact: boolean;
        readonly hasChild: boolean;
        readonly hasAnyChildGranted: boolean;
        readonly hasAllChildGranted: boolean;
        readonly hasAllChildDenied: boolean;
    };
    readonly export: {
        readonly all: {
            scope: string;
            desc: string;
        }[];
        readonly granted: {
            scope: string;
            desc: string;
        }[];
        readonly denied: {
            scope: string;
            desc: string;
        }[];
    };
    readonly granted: {
        scope: string;
        desc: string;
    }[];
    readonly denied: {
        scope: string;
        desc: string;
    }[];
    readonly nodes: TransformSchema<T, PermitNode>;
    readonly can: TransformSchema<T, boolean>;
};
type SchemaType<T> = T extends AccessSchema<infer S> ? S : never;

export { AccessSchema, DataRecord, DataType, Inferer, Schema, SchemaNode, SchemaOptions, SchemaType, UnionScopes, createPermit, createSchema, infer };
