import { AccessSchema, SchemaNode } from './schema';
import { Schema, SchemaOptions, UnionScopes } from './types';

/**
 * Create access control schema
 *
 * @param {T} schema
 * @param {Partial<SchemaOptions>} [options={}]
 * @return {AccessSchema} access schema
 */
export function createSchema<T extends Schema>(
  schema: T,
  options: Partial<SchemaOptions> = {}
) {
  return new AccessSchema(schema, options);
}

type PermitOptions = {
  granted: string[] | SchemaNode[];
  denied: string[] | SchemaNode[];
} & SchemaOptions;

/**
 * Create user permit
 *
 * @export
 * @template T
 * @param {AccessSchema<T>} schema
 * @param {(string[] | SchemaNode[])} scopes
 * @return {Permit}
 */
export function createPermit<T extends Schema>(
  schema: T,
  options: Partial<PermitOptions> = {}
) {
  const schemaObject = new AccessSchema(schema, options);
  const permit = schemaObject.permit(options.granted ?? []);
  if (options.denied instanceof Array) {
    permit.deny(...options.denied);
  }

  return permit;
}

type SchemaType<T> = T extends AccessSchema<infer S> ? S : never;

export type {
  AccessSchema,
  Schema,
  SchemaType,
  SchemaNode,
  SchemaOptions,
  UnionScopes,
};
